@@ -12,23 +12,17 @@ import platform
 import re
 import signal
 import tempfile
-# from getpass import getuser
 from json import dumps as json_dumps
 from json import load as json_load
 from json import loads as json_loads
 from os import environ, path  # , stat
-# from pathlib import Path
 from random import choice, sample, shuffle
 from re import sub
-# from shlex import split as shplit
-# from shutil import which
-from stat import S_IRUSR, S_IWUSR, S_IXUSR, filemode
 from subprocess import CalledProcessError
 from sys import exit as sysexit
 from sys import stdin, stdout, stderr
 from xml.sax.xmlreader import InputSource
 
-# importing this causes the 'config' subcommand to be available
 from jwt.exceptions import PyJWTError
 from pygments import highlight
 from pygments.formatters import Terminal256Formatter
@@ -108,7 +102,7 @@ def main(cli):
 
 
 @cli.argument('-e', '--eval', help="source or eval output to configure shell environment with a login token", arg_only=True, action="store_true", default=False)
-@cli.subcommand('login to a management API')
+@cli.subcommand('login to NetFoundry with a user token or API account credentials')
 def login(cli):
     """Login to an API and cache the expiring token."""
     # if logging in to a NF org (default)
@@ -180,14 +174,35 @@ def login(cli):
                 else:
                     cli.echo(json_dumps(summary_object, indent=4))
         else:             # if eval
+            nonf = """
+# helper function logs out from NetFoundry
+function nonf(){
+    unset   NETFOUNDRY_API_ACCOUNT NETFOUNDRY_API_TOKEN \
+            NETFOUNDRY_CLIENT_ID NETFOUNDRY_PASSWORD NETFOUNDRY_OAUTH_URL \
+            NETFOUNDRY_ORGANIZATION NETFOUNDRY_NETWORK NETFOUNDRY_NETWORK_GROUP \
+            MOPENV MOPURL
+}
+"""
+            noaws = """
+# helper function logs out from AWS
+function noaws(){
+    unset   AWS_SECURITY_TOKEN AWS_SESSION_TOKEN \
+            AWS_ACCESS_KEY AWS_ACCESS_KEY_ID AWS_SECRET_ACCESS_KEY \
+            AWS_REGION AWS_DEFAULT_REGION AWS_SHARED_CREDENTIALS_FILE
+}
+"""
             token_env = f"""
-# $ source <(nfctl --credentials=credentials.json login --eval)
+# $ eval "$({cli.prog_name} --credentials={organization.credentials} login --eval)"
 export NETFOUNDRY_API_TOKEN="{organization.token}"
 export NETFOUNDRY_API_ACCOUNT="{organization.credentials if hasattr(organization, 'credentials') else ''}"
 export NETFOUNDRY_ORGANIZATION="{organization.id}"
-{'export NETFOUNDRY_NETWORK="'+network.id+'"' if network else ''}
-{'export NETFOUNDRY_NETWORK_GROUP="'+network_group.id+'"' if network_group else ''}
-{'export MOPENV="'+organization.environment+'"' if organization.environment else ''}
+{'export NETFOUNDRY_NETWORK="'+network.id+'"' if network else '# NETFOUNDRY_NETWORK'}
+{'export NETFOUNDRY_NETWORK_GROUP="'+network_group.id+'"' if network_group else '# NETFOUNDRY_NETWORK_GROUP'}
+export MOPENV="{organization.environment}"
+export MOPURL="{organization.audience}"
+eval "$(register-python-argcomplete {cli.prog_name})"
+{nonf}
+{noaws}
 """
             if cli.config.general.color:
                 highlighted = highlight(token_env, bash_lexer, Terminal256Formatter(style=cli.config.general.style))
@@ -196,7 +211,7 @@ export NETFOUNDRY_ORGANIZATION="{organization.id}"
                 cli.echo(token_env)
 
 
-@cli.subcommand('logout current profile from an organization')
+@cli.subcommand('logout your identity for the current current profile')
 def logout(cli):
     """Logout by deleting the cached token."""
     spinner = get_spinner("working")
@@ -351,7 +366,7 @@ def edit(cli):
 @cli.argument('-k', '--keys', arg_only=True, action=StoreListKeys, help="list of keys as a,b,c to print only selected keys (columns)")
 @cli.argument('-a', '--as', dest='accept', arg_only=True, choices=['create'], help="request the as=create alternative form of the resource")
 @cli.argument('resource_type', arg_only=True, help='type of resource', metavar="RESOURCE_TYPE", choices=[choice for group in [[singular(type), RESOURCES[type].abbreviation] for type in RESOURCES.keys()] for choice in group])
-@cli.subcommand('get a single resource by query')
+@cli.subcommand('get a single resource by type and query')
 def get(cli, echo: bool = True, embed='all', spinner: object = None):
     """
     Get a single resource as YAML or JSON.
@@ -363,7 +378,7 @@ def get(cli, echo: bool = True, embed='all', spinner: object = None):
         cli.args.resource_type = singular(RESOURCE_ABBREV[cli.args.resource_type].name)
     if not cli.config.general.verbose and cli.args.output in ["yaml", "json"]:    # don't change level if output=text
         cli.log.setLevel(logging.WARN)                                            # don't emit INFO messages to stdout because they will break deserialization
-    if cli.args.accept and not MUTABLE_NET_RESOURCES.get(cli.args.resource_type):
+    if cli.args.accept and not MUTABLE_NET_RESOURCES.get(plural(cli.args.resource_type)):
         logging.warning("ignoring --as=create becuase it is applicable only to mutable resources in the network domain")
         cli.args['accept'] = None
     match = {}
@@ -532,10 +547,10 @@ def get(cli, echo: bool = True, embed='all', spinner: object = None):
 
 @cli.argument('query', arg_only=True, action=StoreDictKeyPair, nargs='?', help="query params as k=v,k=v comma-separated pairs")
 @cli.argument('-k', '--keys', arg_only=True, action=StoreListKeys, help="list of keys as a,b,c to print only selected keys (columns)")
-@cli.argument('-m', '--mine', arg_only=True, action='store_true', help="filter roles by caller identity")
+@cli.argument('-m', '--my-roles', arg_only=True, action='store_true', help="filter roles by caller identity")
 @cli.argument('-a', '--as', dest='accept', arg_only=True, choices=['create'], help="request the as=create alternative form of the resources")
 @cli.argument('resource_type', arg_only=True, help='type of resource', metavar="RESOURCE_TYPE", choices=[choice for group in [[type, RESOURCES[type].abbreviation] for type in RESOURCES.keys()] for choice in group])
-@cli.subcommand(description='find resources as lists')
+@cli.subcommand(description='find a collection of resources by type and query')
 def list(cli, spinner: object = None):
     """Find resources as lists."""
     if not spinner:
@@ -550,7 +565,7 @@ def list(cli, spinner: object = None):
         cli.log.warn("try 'get' command to get by id")
     if cli.args.output == "text":
         if not stdout.isatty():
-            cli.log.warning("use --output=yaml or json for scripting nfctl")
+            cli.log.warning(f"use --output=yaml or json for scripting {cli.prog_name}")
     else:             # output is YAML or JSON
         # don't emit INFO messages to stdout because they will break deserialization
         cli.log.setLevel(logging.WARN)
@@ -570,9 +585,9 @@ def list(cli, spinner: object = None):
         elif cli.args.resource_type in ["identities", "user-identities", "api-account-identities"]:
             matches = organization.find_identities(type=cli.args.resource_type, **cli.args.query)
         elif cli.args.resource_type == "roles":
-            if cli.args.mine:
+            if cli.args.my_roles:
                 if cli.args.query.get('identityId'):
-                    cli.log.warning("got --mine, ignoring param 'identityId'")
+                    cli.log.warning("got --my-roles, ignoring param 'identityId'")
                 cli.args.query['identityId'] = organization.caller['id']
                 matches = organization.find_roles(**cli.args.query)
             else:
@@ -667,7 +682,7 @@ def list(cli, spinner: object = None):
 
 @cli.argument('query', arg_only=True, action=StoreDictKeyPair, nargs='?', help="query params as k=v,k=v comma-separated pairs")
 @cli.argument('resource_type', arg_only=True, help='type of resource', choices=[choice for group in [[singular(type), RESOURCES[type].abbreviation] for type in RESOURCES.keys()] for choice in group])
-@cli.subcommand('delete a resource in the network domain')
+@cli.subcommand('delete a single resource by type and query')
 def delete(cli):
     """Delete a resource in the network domain."""
     spinner = get_spinner("working")
@@ -751,10 +766,13 @@ def delete(cli):
             sysexit(1)
 
 
+@cli.argument("-p", "--prefix", default=f"{cli.prog_name}-demo", help="choose a network name prefix to identify all of your demos")
+@cli.argument("-j", "--jwt", action="store_boolean", default=True, help="save the one-time enroll token for each demo identity in the current directory")
+@cli.argument("-e", "--echo-name", arg_only=True, action="store_true", default=False, help="only echo a friendly network name then exit")
 @cli.argument("-s", "--size", default="small", help=argparse.SUPPRESS)   # troubleshoot scale-up instance size factor
 @cli.argument("-v", "--product-version", default="default", help="network product version: 'default', 'latest', or any active semver")
-@cli.argument("--provider", default="AWS", required=False, help="cloud provider to host edge routers", choices=DC_PROVIDERS)
-@cli.argument("--regions", dest="regions", default=["us-west-1"], nargs="+", help="cloud location codes in which to host edge routers")
+@cli.argument("--provider", default="AWS", help="cloud provider for hosted edge routers", choices=DC_PROVIDERS)
+@cli.argument("--regions", dest="regions", default=["us-west-1"], nargs="+", help="provider regions for hosted edge routers")
 @cli.subcommand('create a functioning demo network')
 def demo(cli):
     """Create a demo network or add demo resources to existing network."""
@@ -764,11 +782,14 @@ def demo(cli):
     if cli.config.general.network:
         network_name = cli.config.general.network
     else:
-        resources_dir = path.join(path.dirname(__file__), 'resources')
-        friendly_words_filename = path.join(resources_dir, "friendly-words/generated/words.json")
+        friendly_words_dir = path.join(path.dirname(__file__), 'friendly-words')
+        friendly_words_filename = path.join(friendly_words_dir, "generated/words.json")
         with open(friendly_words_filename, 'r') as friendly_words_path:
             friendly_words = json_load(friendly_words_path)
-        network_name = f"nfctl-demo-{choice(friendly_words['predicates'])}-{choice(friendly_words['objects'])}"
+        network_name = f"{cli.config.demo.prefix}-{choice(friendly_words['predicates'])}-{choice(friendly_words['objects'])}"
+        if cli.args.echo_name:
+            cli.echo(network_name)
+            sysexit(0)
     demo_confirmed = False
     if cli.config.general.yes:
         demo_confirmed = True
@@ -806,7 +827,7 @@ def demo(cli):
                 name=network_name,
                 size=cli.config.demo.size,
                 version=cli.config.demo.product_version,
-                wait=0)  # don't use wait > 0 until process-executions beta is launched, until then poll for status
+                wait=0)  # FIXME: don't use wait > 0 until process-executions beta is launched, until then poll for status
             network, network_group = use_network(
                 organization=organization,
                 group=cli.config.general.network_group,
@@ -901,17 +922,22 @@ def demo(cli):
         endpoints[exit] = {
             "attributes": ["#demo_exits"]
         }
-    for end in endpoints.keys():
+    for end, spec in endpoints.items():
         spinner.text = f"Finding endpoint '{end}'"
         with spinner:
             if not network.endpoint_exists(name=end):
                 # create an endpoint for the dialing device that will access services
                 spinner.text = f"Creating endpoint '{end}'"
-                endpoints[end]['properties'] = network.create_endpoint(name=end, attributes=endpoints[end]['attributes'])
+                endpoints[end]['properties'] = network.create_endpoint(name=end, attributes=spec['attributes'])
                 spinner.succeed(sub('Creating', 'Created', spinner.text))
             else:
-                endpoints[end]['properties'] = network.endpoints(name=end)[0]
+                spec['properties'] = network.endpoints(name=end)[0]
                 spinner.succeed(sub('Finding', 'Found', spinner.text))
+            if cli.args.jwt and spec['properties'].get('jwt'):  # save if --jwt, token is absent if already enrolled
+                jwt = spec['properties']['jwt']
+                with open(end+'.jwt', 'wt') as tf:
+                    tf.write(jwt)
+                    cli.log.info(f"Saved enrollment token in {tf.name}")
 
     services = {
         "Fireworks Service": {
